#
# This is an example configuration which loads the documentation classes for
# the webtools package and shows you how to configure the various different 
# classes available. Your application should have it's own configuration in it's
# CVS area and not use this, other than as a guideline. Applications deployed at
# CERN will need to meet the CERN web service SLA:
#     https://twiki.cern.ch/twiki/bin/view/CMS/DMWTServiceLevelAgreement 
# This includes committing configuration files to appropriate locations in CVS.
#
from WMCore.Configuration import Configuration
from os import environ

config = Configuration()

# This component has all the configuration of CherryPy
config.component_('Webtools')
# We could change port, set logging etc here like so:
#config.Webtools.port = 8011
#config.Webtools.environment = development
# etc. Check Root.py for all configurables

# This is the application
config.Webtools.application = 'WebtoolsDocs'

# This is the config for the application
config.component_('WebtoolsDocs')
# Define the default location for templates for the app
config.WebtoolsDocs.templates = environ['WTBASE'] + '/templates/WMCore/WebTools'
config.WebtoolsDocs.admin = 'your@email.com'
config.WebtoolsDocs.title = 'CMS WMCore/WebTools Documentation'
config.WebtoolsDocs.description = 'Documentation on the WMCORE/WebTools'

# We could define the class that is the applications index
#config.WebtoolsDocs.index = 'welcome'
# but instead we'll leave it blank and use the default (Welcome.py) which 
# inspects the pages that are loaded and auto-generates a page based on the 
# classes doc strings. You can hide a view from the welcome page by setting 
# hidden=True in its configuration - useful for "admin" pages.

# Views are all pages 
config.WebtoolsDocs.section_('views')
# These are all the active pages that Root.py should instantiate 
active = config.WebtoolsDocs.views.section_('active')
# The section name is also the location the class will be located
# e.g. http://localhost:8080/documentation
active.section_('documentation')
# The class to load for this view/page
active.documentation.object = 'WMCore.WebTools.Documentation'
# I could add a variable to the documenation object if I wanted to as follows:
# active.documentation.foo = 'bar'

# I can reuse a class
active.section_('secretdocumentation')
active.secretdocumentation.object = 'WMCore.WebTools.Documentation'
# I don't want the world to see the secret documents on the welcome page. 
active.secretdocumentation.hidden = True

#active.section_('welcome')
#active.welcome.object = 'WMCore.WebTools.Welcome'

# Controllers are standard way to return minified gzipped css and js
active.section_('controllers')
# The class to load for this view/page
active.controllers.object = 'WMCore.WebTools.Controllers'
# The configuration for this object - the location of css and js
active.controllers.css = {'reset.css': environ['YUI_ROOT'] + '/reset/reset.css', 
        'cms_reset.css': environ['WMCORE_ROOT'] + '/src/css/WMCore/WebTools/cms_reset.css', 
        'style.css': environ['WMCORE_ROOT'] + '/src/css/WMCore/WebTools/style.css'}
active.controllers.js = {}
# These are pages in "maintenance mode" - to be completed
maint = config.WebtoolsDocs.views.section_('maintenance')

active.section_('masthead')
active.masthead.object = 'WMCore.WebTools.Masthead'
active.masthead.templates = environ['WTBASE'] + '/templates/WMCore/WebTools/Masthead'

# This is how you would configure a RESTful service, commented out because 
# you'll need an active database for it to make sense - we just want to show the
# documentation 
#active.section_('rest')
#active.rest.object = 'WMCore.WebTools.RESTApi'
#active.rest.templates = environ['WTBASE'] + '/templates/WMCore/WebTools/'
##active.rest.database = 'sqlite:////Users/metson/Documents/Workspace/GenDB/gendb.lite'
#active.rest.section_('model')
#active.rest.model.object = 'RESTModel'
#active.rest.model.database = 'sqlite:////Users/metson/Documents/Workspace/GenDB/gendb.lite'
#active.rest.model.templates = environ['WTBASE'] + '/templates/WMCore/WebTools/'
#active.rest.section_('formatter')
#active.rest.formatter.object = 'RESTFormatter'
#active.rest.formatter.templates = environ['WTBASE'] + '/templates/WMCore/WebTools/'
